@@ -1,22 +1,22 @@
 /* data.c - An abstraction for data objects.
-   Copyright (C) 2002, 2003, 2004, 2005, 2007 g10 Code GmbH
-
-   This file is part of GPGME.
-
-   GPGME is free software; you can redistribute it and/or modify it
-   under the terms of the GNU Lesser General Public License as
-   published by the Free Software Foundation; either version 2.1 of
-   the License, or (at your option) any later version.
-
-   GPGME is distributed in the hope that it will be useful, but
-   WITHOUT ANY WARRANTY; without even the implied warranty of
-   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
-   Lesser General Public License for more details.
-
-   You should have received a copy of the GNU Lesser General Public
-   License along with this program; if not, write to the Free Software
-   Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
-   02111-1307, USA.  */
+ * Copyright (C) 2002, 2003, 2004, 2005, 2007 g10 Code GmbH
+ *
+ * This file is part of GPGME.
+ *
+ * GPGME is free software; you can redistribute it and/or modify it
+ * under the terms of the GNU Lesser General Public License as
+ * published by the Free Software Foundation; either version 2.1 of
+ * the License, or (at your option) any later version.
+ *
+ * GPGME is distributed in the hope that it will be useful, but
+ * WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
+ * Lesser General Public License for more details.
+ *
+ * You should have received a copy of the GNU Lesser General Public
+ * License along with this program; if not, see <https://gnu.org/licenses/>.
+ * SPDX-License-Identifier: LGPL-2.1-or-later
+ */
 
 #if HAVE_CONFIG_H
 #include <config.h>
@@ -52,7 +52,7 @@
  */
 struct property_s
 {
-  gpgme_data_t dh;   /* The data objcet or NULL if the slot is not used.  */
+  gpgme_data_t dh;   /* The data object or NULL if the slot is not used.  */
   uint64_t dserial;  /* The serial number of the data object.  */
   struct {
     unsigned int blankout : 1;  /* Void the held data.  */
@@ -178,7 +178,7 @@ _gpgme_data_set_prop (gpgme_data_t dh, uint64_t dserial,
 {
   gpg_error_t err = 0;
   int idx;
-  TRACE_BEG3 (DEBUG_DATA, "gpgme_data_set_prop", dh,
+  TRACE_BEG  (DEBUG_DATA, "gpgme_data_set_prop", dh,
 	      "dserial=%llu %lu=%d",
               (unsigned long long)dserial,
               (unsigned long)name, value);
@@ -241,7 +241,7 @@ _gpgme_data_get_prop (gpgme_data_t dh, uint64_t dserial,
 {
   gpg_error_t err = 0;
   int idx;
-  TRACE_BEG2 (DEBUG_DATA, "gpgme_data_get_prop", dh,
+  TRACE_BEG  (DEBUG_DATA, "gpgme_data_get_prop", dh,
 	      "dserial=%llu %lu",
               (unsigned long long)dserial,
               (unsigned long)name);
@@ -339,6 +339,21 @@ _gpgme_data_release (gpgme_data_t dh)
   remove_from_property_table (dh, dh->propidx);
   if (dh->file_name)
     free (dh->file_name);
+  if (dh->inbound_buffer)
+    {
+      if (dh->sensitive)
+        _gpgme_wipememory (dh->inbound_buffer, dh->io_buffer_size);
+      free (dh->inbound_buffer);
+    }
+  if (dh->outbound_buffer)
+    {
+      if (dh->sensitive)
+        _gpgme_wipememory (dh->outbound_buffer, dh->io_buffer_size);
+      free (dh->outbound_buffer);
+    }
+  if (dh->sensitive)
+    _gpgme_wipememory (dh->outboundspace, BUFFER_SIZE);
+
   free (dh);
 }
 
@@ -352,8 +367,8 @@ gpgme_data_read (gpgme_data_t dh, void *buffer, size_t size)
 {
   gpgme_ssize_t res;
   int blankout;
-  TRACE_BEG2 (DEBUG_DATA, "gpgme_data_read", dh,
-	      "buffer=%p, size=%u", buffer, size);
+  TRACE_BEG  (DEBUG_DATA, "gpgme_data_read", dh,
+	      "buffer=%p, size=%zu", buffer, size);
 
   if (!dh)
     {
@@ -376,7 +391,7 @@ gpgme_data_read (gpgme_data_t dh, void *buffer, size_t size)
       while (res < 0 && errno == EINTR);
     }
 
-  return TRACE_SYSRES (res);
+  return TRACE_SYSRES ((int)res);
 }
 
 
@@ -387,8 +402,8 @@ gpgme_ssize_t
 gpgme_data_write (gpgme_data_t dh, const void *buffer, size_t size)
 {
   gpgme_ssize_t res;
-  TRACE_BEG2 (DEBUG_DATA, "gpgme_data_write", dh,
-	      "buffer=%p, size=%u", buffer, size);
+  TRACE_BEG  (DEBUG_DATA, "gpgme_data_write", dh,
+	      "buffer=%p, size=%zu", buffer, size);
 
   if (!dh)
     {
@@ -404,18 +419,18 @@ gpgme_data_write (gpgme_data_t dh, const void *buffer, size_t size)
     res = (*dh->cbs->write) (dh, buffer, size);
   while (res < 0 && errno == EINTR);
 
-  return TRACE_SYSRES (res);
+  return TRACE_SYSRES ((int)res);
 }
 
 
 /* Set the current position from where the next read or write starts
-   in the data object with the handle DH to OFFSET, relativ to
+   in the data object with the handle DH to OFFSET, relative to
    WHENCE.  */
 gpgme_off_t
 gpgme_data_seek (gpgme_data_t dh, gpgme_off_t offset, int whence)
 {
-  TRACE_BEG2 (DEBUG_DATA, "gpgme_data_seek", dh,
-	      "offset=%lli, whence=%i", offset, whence);
+  TRACE_BEG  (DEBUG_DATA, "gpgme_data_seek", dh,
+	      "offset=%lli, whence=%i", (long long int)offset, whence);
 
   if (!dh)
     {
@@ -431,13 +446,13 @@ gpgme_data_seek (gpgme_data_t dh, gpgme_off_t offset, int whence)
   /* For relative movement, we must take into account the actual
      position of the read counter.  */
   if (whence == SEEK_CUR)
-    offset -= dh->pending_len;
+    offset -= dh->outbound_pending;
 
   offset = (*dh->cbs->seek) (dh, offset, whence);
   if (offset >= 0)
-    dh->pending_len = 0;
+    dh->outbound_pending = 0;
 
-  return TRACE_SYSRES (offset);
+  return TRACE_SYSRES ((int)offset);
 }
 
 
@@ -446,7 +461,7 @@ gpgme_error_t
 gpgme_data_rewind (gpgme_data_t dh)
 {
   gpgme_error_t err;
-  TRACE_BEG (DEBUG_DATA, "gpgme_data_rewind", dh);
+  TRACE_BEG  (DEBUG_DATA, "gpgme_data_rewind", dh, "");
 
   err = ((gpgme_data_seek (dh, 0, SEEK_SET) == -1)
          ? gpg_error_from_syserror () : 0);
@@ -459,7 +474,7 @@ gpgme_data_rewind (gpgme_data_t dh)
 void
 gpgme_data_release (gpgme_data_t dh)
 {
-  TRACE (DEBUG_DATA, "gpgme_data_release", dh);
+  TRACE (DEBUG_DATA, "gpgme_data_release", dh, "");
 
   if (!dh)
     return;
@@ -475,8 +490,8 @@ gpgme_data_release (gpgme_data_t dh)
 gpgme_data_encoding_t
 gpgme_data_get_encoding (gpgme_data_t dh)
 {
-  TRACE1 (DEBUG_DATA, "gpgme_data_get_encoding", dh,
-	  "dh->encoding=%i", dh ? dh->encoding : GPGME_DATA_ENCODING_NONE);
+  TRACE (DEBUG_DATA, "gpgme_data_get_encoding", dh,
+         "dh->encoding=%i", dh ? dh->encoding : GPGME_DATA_ENCODING_NONE);
   return dh ? dh->encoding : GPGME_DATA_ENCODING_NONE;
 }
 
@@ -486,7 +501,7 @@ gpgme_data_get_encoding (gpgme_data_t dh)
 gpgme_error_t
 gpgme_data_set_encoding (gpgme_data_t dh, gpgme_data_encoding_t enc)
 {
-  TRACE_BEG1 (DEBUG_DATA, "gpgme_data_set_encoding", dh,
+  TRACE_BEG  (DEBUG_DATA, "gpgme_data_set_encoding", dh,
 	      "encoding=%i", enc);
   if (!dh)
     return TRACE_ERR (gpg_error (GPG_ERR_INV_VALUE));
@@ -502,7 +517,7 @@ gpgme_data_set_encoding (gpgme_data_t dh, gpgme_data_encoding_t enc)
 gpgme_error_t
 gpgme_data_set_file_name (gpgme_data_t dh, const char *file_name)
 {
-  TRACE_BEG1 (DEBUG_DATA, "gpgme_data_set_file_name", dh,
+  TRACE_BEG  (DEBUG_DATA, "gpgme_data_set_file_name", dh,
 	      "file_name=%s", file_name);
 
   if (!dh)
@@ -531,12 +546,12 @@ gpgme_data_get_file_name (gpgme_data_t dh)
 {
   if (!dh)
     {
-      TRACE (DEBUG_DATA, "gpgme_data_get_file_name", dh);
+      TRACE (DEBUG_DATA, "gpgme_data_get_file_name", dh, "");
       return NULL;
     }
 
-  TRACE1 (DEBUG_DATA, "gpgme_data_get_file_name", dh,
-	  "dh->file_name=%s", dh->file_name);
+  TRACE (DEBUG_DATA, "gpgme_data_get_file_name", dh,
+         "dh->file_name=%s", dh->file_name);
   return dh->file_name;
 }
 
@@ -545,7 +560,7 @@ gpgme_data_get_file_name (gpgme_data_t dh)
 gpg_error_t
 gpgme_data_set_flag (gpgme_data_t dh, const char *name, const char *value)
 {
-  TRACE_BEG2 (DEBUG_DATA, "gpgme_data_set_flag", dh,
+  TRACE_BEG  (DEBUG_DATA, "gpgme_data_set_flag", dh,
 	      "%s=%s", name, value);
 
   if (!dh)
@@ -555,6 +570,28 @@ gpgme_data_set_flag (gpgme_data_t dh, const char *name, const char *value)
     {
       dh->size_hint= value? _gpgme_string_to_off (value) : 0;
     }
+  else if (!strcmp (name, "io-buffer-size"))
+    {
+      gpgme_off_t val;
+
+      /* We may set this only once.  */
+      if (dh->io_buffer_size)
+        return gpg_error (GPG_ERR_CONFLICT);
+
+      val = value? _gpgme_string_to_off (value) : 0;
+      if (val > 1024*1024)
+        val = 1024*1024;  /* Cap at 1MiB */
+      else if (val < BUFFER_SIZE)
+        val = 0;          /* We can use the default buffer.  */
+
+      /* Actual allocation happens as needed but we round it to a
+       * multiple of 1k. */
+      dh->io_buffer_size = ((val + 1023)/1024)*1024;
+    }
+  else if (!strcmp (name, "sensitive"))
+    {
+      dh->sensitive = (value && *value)? !!atoi (value) : 0;
+    }
   else
     return gpg_error (GPG_ERR_UNKNOWN_NAME);
 
@@ -569,14 +606,35 @@ gpgme_error_t
 _gpgme_data_inbound_handler (void *opaque, int fd)
 {
   struct io_cb_data *data = (struct io_cb_data *) opaque;
+  gpg_error_t err;
   gpgme_data_t dh = (gpgme_data_t) data->handler_value;
-  char buffer[BUFFER_SIZE];
-  char *bufp = buffer;
+  char bufferspace[BUFFER_SIZE];
+  char *buffer;
+  size_t buffer_size;
+  char *bufp;
   gpgme_ssize_t buflen;
-  TRACE_BEG1 (DEBUG_CTX, "_gpgme_data_inbound_handler", dh,
-	      "fd=0x%x", fd);
+  TRACE_BEG  (DEBUG_CTX, "_gpgme_data_inbound_handler", dh,
+	      "fd=%d", fd);
 
-  buflen = _gpgme_io_read (fd, buffer, BUFFER_SIZE);
+  if (dh->io_buffer_size)
+    {
+      if (!dh->inbound_buffer)
+        {
+          dh->inbound_buffer = malloc (dh->io_buffer_size);
+          if (!dh->inbound_buffer)
+            return TRACE_ERR (gpg_error_from_syserror ());
+        }
+      buffer_size = dh->io_buffer_size;
+      buffer = dh->inbound_buffer;
+    }
+  else
+    {
+      buffer_size = BUFFER_SIZE;
+      buffer = bufferspace;
+    }
+  bufp = buffer;
+
+  buflen = _gpgme_io_read (fd, buffer, buffer_size);
   if (buflen < 0)
     return gpg_error_from_syserror ();
   if (buflen == 0)
@@ -589,12 +647,21 @@ _gpgme_data_inbound_handler (void *opaque, int fd)
     {
       gpgme_ssize_t amt = gpgme_data_write (dh, bufp, buflen);
       if (amt == 0 || (amt < 0 && errno != EINTR))
-	return TRACE_ERR (gpg_error_from_syserror ());
+	{
+          err = gpg_error_from_syserror ();
+          goto leave;
+        }
       bufp += amt;
       buflen -= amt;
     }
   while (buflen > 0);
-  return TRACE_ERR (0);
+  err = 0;
+
+ leave:
+  if (dh->sensitive && buffer == bufferspace)
+    _gpgme_wipememory (bufferspace, BUFFER_SIZE);
+
+  return TRACE_ERR (err);
 }
 
 
@@ -603,13 +670,34 @@ _gpgme_data_outbound_handler (void *opaque, int fd)
 {
   struct io_cb_data *data = (struct io_cb_data *) opaque;
   gpgme_data_t dh = (gpgme_data_t) data->handler_value;
+  char *buffer;
+  size_t buffer_size;
   gpgme_ssize_t nwritten;
-  TRACE_BEG1 (DEBUG_CTX, "_gpgme_data_outbound_handler", dh,
-	      "fd=0x%x", fd);
+  TRACE_BEG  (DEBUG_CTX, "_gpgme_data_outbound_handler", dh,
+	      "fd=%d", fd);
 
-  if (!dh->pending_len)
+  if (dh->io_buffer_size)
     {
-      gpgme_ssize_t amt = gpgme_data_read (dh, dh->pending, BUFFER_SIZE);
+      if (!dh->outbound_buffer)
+        {
+          dh->outbound_buffer = malloc (dh->io_buffer_size);
+          if (!dh->outbound_buffer)
+            return TRACE_ERR (gpg_error_from_syserror ());
+          dh->outbound_pending = 0;
+        }
+      buffer_size = dh->io_buffer_size;
+      buffer = dh->outbound_buffer;
+    }
+  else
+    {
+      buffer_size = BUFFER_SIZE;
+      buffer = dh->outboundspace;
+    }
+
+
+  if (!dh->outbound_pending)
+    {
+      gpgme_ssize_t amt = gpgme_data_read (dh, buffer, buffer_size);
       if (amt < 0)
 	return TRACE_ERR (gpg_error_from_syserror ());
       if (amt == 0)
@@ -617,10 +705,10 @@ _gpgme_data_outbound_handler (void *opaque, int fd)
 	  _gpgme_io_close (fd);
 	  return TRACE_ERR (0);
 	}
-      dh->pending_len = amt;
+      dh->outbound_pending = amt;
     }
 
-  nwritten = _gpgme_io_write (fd, dh->pending, dh->pending_len);
+  nwritten = _gpgme_io_write (fd, buffer, dh->outbound_pending);
   if (nwritten == -1 && errno == EAGAIN)
     return TRACE_ERR (0);
 
@@ -637,9 +725,9 @@ _gpgme_data_outbound_handler (void *opaque, int fd)
   if (nwritten <= 0)
     return TRACE_ERR (gpg_error_from_syserror ());
 
-  if (nwritten < dh->pending_len)
-    memmove (dh->pending, dh->pending + nwritten, dh->pending_len - nwritten);
-  dh->pending_len -= nwritten;
+  if (nwritten < dh->outbound_pending)
+    memmove (buffer, buffer + nwritten, dh->outbound_pending - nwritten);
+  dh->outbound_pending -= nwritten;
   return TRACE_ERR (0);
 }
 
