@@ -258,11 +258,7 @@ gpgme_get_engine_info (gpgme_engine_info_t *info)
           if (!err && version && engine_minimal_version
               && !_gpgme_compare_versions (version, engine_minimal_version))
             {
-#if GPG_ERROR_VERSION_NUMBER < 0x011900 /* 1.25 */
-              err = gpg_error (GPG_ERR_NO_ENGINE);
-#else
               err = gpg_error (GPG_ERR_ENGINE_TOO_OLD);
-#endif
             }
 
           /* Now set the dummy version for pseudo engines.  */
@@ -824,6 +820,20 @@ _gpgme_engine_op_keysign (engine_t engine, gpgme_key_t key, const char *userid,
 }
 
 
+gpgme_error_t
+_gpgme_engine_op_revsig (engine_t engine, gpgme_key_t key, gpgme_key_t signing_key,
+                         const char *userid, unsigned int flags)
+{
+  if (!engine)
+    return gpg_error (GPG_ERR_INV_VALUE);
+
+  if (!engine->ops->revsig)
+    return gpg_error (GPG_ERR_NOT_IMPLEMENTED);
+
+  return (*engine->ops->revsig) (engine->engine, key, signing_key, userid, flags);
+}
+
+
 gpgme_error_t
 _gpgme_engine_op_tofu_policy (engine_t engine,
                               gpgme_key_t key,  gpgme_tofu_policy_t policy)
@@ -1128,3 +1138,17 @@ _gpgme_engine_op_spawn (engine_t engine,
   return (*engine->ops->opspawn) (engine->engine, file, argv,
                                   datain, dataout, dataerr, flags);
 }
+
+gpgme_error_t
+_gpgme_engine_op_setexpire (engine_t engine, gpgme_key_t key,
+                            unsigned long expires, const char *subfprs,
+                            unsigned int reserved)
+{
+  if (!engine)
+    return gpg_error (GPG_ERR_INV_VALUE);
+
+  if (!engine->ops->setexpire)
+    return gpg_error (GPG_ERR_NOT_IMPLEMENTED);
+
+  return (*engine->ops->setexpire) (engine->engine, key, expires, subfprs, reserved);
+}
