@@ -23,12 +23,8 @@
 #define UTIL_H
 
 #ifdef HAVE_W32_SYSTEM
-# ifdef HAVE_W32CE_SYSTEM
-#  include "w32-ce.h"
-# else
-#  include "winsock2.h"
-#  include "windows.h"
-# endif
+# include "winsock2.h"
+# include "windows.h"
 #endif
 
 /* For pid_t.  */
@@ -45,15 +41,6 @@
 
 #define DIM(v) (sizeof(v)/sizeof((v)[0]))
 
-#if GPG_ERROR_VERSION_NUMBER < 0x011900 /* 1.25 */
-# define GPG_ERR_ENGINE_TOO_OLD 300
-# define GPG_ERR_TOO_OLD        308
-#endif
-
-#ifndef GPGRT_ATTR_SENTINEL
-# define GPGRT_ATTR_SENTINEL(a)  /* */
-#endif
-
 
 
 /*-- {posix,w32}-util.c --*/
@@ -109,6 +96,9 @@ int _gpgme_ttyname_r (int fd, char *buf, size_t buflen);
 
 /*-- conversion.c --*/
 
+/* Make sure to to erase the memory (PTR,LEN).  */
+void _gpgme_wipememory (void *ptr, size_t len);
+
 /* Concatenate the string S1 with all the following strings up to a
    NULL.  Returns a malloced buffer with the new string or NULL on a
    malloc error or if too many arguments are given.  */
@@ -165,10 +155,11 @@ time_t _gpgme_parse_timestamp (const char *timestamp, char **endp);
  * on error or missing timestamp.  */
 unsigned long _gpgme_parse_timestamp_ul (const char *timestamp);
 
-gpgme_error_t _gpgme_map_gnupg_error (char *err);
-
 int _gpgme_map_pk_algo (int algo, gpgme_protocol_t protocol);
 
+const char *_gpgme_cipher_algo_name (int algo, gpgme_protocol_t protocol);
+const char *_gpgme_cipher_mode_name (int algo, gpgme_protocol_t protocol);
+
 
 /*-- b64dec.c --*/
 
@@ -207,9 +198,6 @@ const char *_gpgme_status_to_string (gpgme_status_code_t code);
 int _gpgme_mkstemp (int *fd, char **name);
 const char *_gpgme_get_w32spawn_path (void);
 #endif /*HAVE_W32_SYSTEM*/
-#ifdef HAVE_W32CE_SYSTEM
-char *_gpgme_w32ce_get_debug_envvar (void);
-#endif /*HAVE_W32CE_SYSTEM*/
 
 
 
